==> racer-0.0.6.tgz: package/lib/Store.js <==
var MAX_RETRIES, MemoryAdapter, Model, PubSub, RETRY_DELAY, Stm, Store, TxnApplier, pathParser, redis, redisInfo, transaction;
var __slice = Array.prototype.slice;
redis = require('redis');
MemoryAdapter = require('./adapters/Memory');
Model = require('./Model');
Stm = require('./Stm');
PubSub = require('./PubSub');
transaction = require('./transaction');
TxnApplier = require('./TxnApplier');
pathParser = require('./pathParser.server');
redisInfo = require('./redisInfo');
MAX_RETRIES = 20;
RETRY_DELAY = 5;
Store = module.exports = function(AdapterClass, options) {
  var StoreAtomic, adapter, clientId, clientSockets, commit, db, forTxnSince, hasInvalidVer, host, ignoreSubscribe, localModels, nextClientId, nextTxnId, nextTxnNum, populateModel, port, pubSub, redisClient, redisOptions, redisStarts, self, setModelValue, startId, stm, subClient, subscribeToStarts, txnApplier, txnCount, txnSubClient, _ref;
  if (AdapterClass == null) {
    AdapterClass = MemoryAdapter;
  }
  if (options == null) {
    options = {};
  }
  self = this;
  this._adapter = adapter = new AdapterClass;
  _ref = redisOptions = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db;
  this._redisClient = redisClient = redis.createClient(port, host, redisOptions);
  this._subClient = subClient = redis.createClient(port, host, redisOptions);
  this._txnSubClient = txnSubClient = redis.createClient(port, host, redisOptions);
  redisStarts = null;
  startId = null;
  ignoreSubscribe = false;
  (subscribeToStarts = function(selected) {
    if (ignoreSubscribe) {
      return ignoreSubscribe = false;
    }
    if (db !== void 0 && !selected) {
      return redisClient.select(db, function(err) {
        if (err) {
          throw err;
        }
        return subscribeToStarts(true);
      });
    }
    return redisInfo.subscribeToStarts(subClient, redisClient, function(starts) {
      redisStarts = starts;
      return startId = starts[0][0];
    });
  })();
  ignoreSubscribe = true;
  redisClient.on('connect', subscribeToStarts);
  redisClient.on('end', function() {
    redisStarts = null;
    return startId = null;
  });
  this._localModels = localModels = {};
  this._pubSub = pubSub = new PubSub({
    redis: redisOptions,
    pubClient: redisClient,
    subClient: txnSubClient,
    onMessage: function(clientId, txn) {
      var base, model, socket;
      if (clientId === transaction.clientId(txn)) {
        return;
      }
      if (model = localModels[clientId]) {
        return model._onTxn(txn);
      }
      if (socket = clientSockets[clientId]) {
        base = transaction.base(txn);
        if (base > socket.__base) {
          socket.__base = base;
          return nextTxnNum(clientId, function(num) {
            return socket.emit('txn', txn, num);
          });
        }
      }
    }
  });
  this._nextTxnNum = nextTxnNum = function(clientId, callback) {
    return redisClient.incr('txnClock.' + clientId, function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value);
    });
  };
  hasInvalidVer = function(socket, ver, clientStartId) {
    if (!startId) {
      socket.disconnect();
      return true;
    }
    if (!(clientStartId && clientStartId === startId)) {
      socket.emit('fatalErr');
      return true;
    }
    return false;
  };
  clientSockets = {};
  this._setSockets = function(sockets) {
    return sockets.on('connection', function(socket) {
      return socket.on('sub', function(clientId, paths, ver, clientStartId) {
        var txnsSince;
        if (hasInvalidVer(socket, ver, clientStartId)) {
          return;
        }
        socket.on('disconnect', function() {
          pubSub.unsubscribe(clientId);
          delete clientSockets[clientId];
          return redisClient.del('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
          });
        });
        socket.on('txn', function(txn, clientStartId) {
          var base;
          base = transaction.base(txn);
          if (hasInvalidVer(socket, base, clientStartId)) {
            return;
          }
          return commit(txn, function(err, txn) {
            var txnId;
            txnId = transaction.id(txn);
            base = transaction.base(txn);
            if (err && err !== 'duplicate') {
              return socket.emit('txnErr', err, txnId);
            }
            return nextTxnNum(clientId, function(num) {
              return socket.emit('txnOk', txnId, base, num);
            });
          });
        });
        socket.on('txnsSince', txnsSince = function(ver, clientStartId) {
          if (hasInvalidVer(socket, ver, clientStartId)) {
            return;
          }
          return redisClient.get('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
            socket.emit('txnNum', value || 0);
            return forTxnSince(ver, clientId, function(txn) {
              return nextTxnNum(clientId, function(num) {
                socket.__base = transaction.base(txn);
                return socket.emit('txn', txn, num);
              });
            });
          });
        });
        socket.__base = 0;
        clientSockets[clientId] = socket;
        pubSub.subscribe(clientId, paths);
        return txnsSince(ver + 1, clientStartId);
      });
    });
  };
  this._forTxnSince = forTxnSince = function(ver, clientId, onTxn, done) {
    if (!pubSub.hasSubscriptions(clientId)) {
      return;
    }
    return redisClient.zrangebyscore('txns', ver, '+inf', 'withscores', function(err, vals) {
      var i, txn, val, _len;
      if (err) {
        throw err;
      }
      txn = null;
      for (i = 0, _len = vals.length; i < _len; i++) {
        val = vals[i];
        if (i % 2) {
          if (!pubSub.subscribedToTxn(clientId, txn)) {
            continue;
          }
          transaction.base(txn, +val);
          onTxn(txn);
        } else {
          txn = JSON.parse(val);
        }
      }
      if (done) {
        return done();
      }
    });
  };
  setModelValue = function(modelAdapter, root, remainder, value, ver) {
    var appendRoot, c, nextRoot, nextValue, prop, _ref2, _results;
    if (!remainder) {
      value = typeof value === 'object' ? Array.isArray(value) ? [] : {} : value;
      return modelAdapter.set(root, value, ver);
    }
    if (remainder === '**') {
      return modelAdapter.set(root, value, ver);
    }
    if (remainder.charAt(0) === '*' && (c = remainder.charAt(1)) === '.' || c === '') {
      _ref2 = pathParser.splitPattern(remainder.substr(2)), appendRoot = _ref2[0], remainder = _ref2[1];
      _results = [];
      for (prop in value) {
        nextRoot = root ? root + '.' + prop : prop;
        nextValue = value[prop];
        if (appendRoot) {
          nextRoot += '.' + appendRoot;
          nextValue = pathParser.fastLookup(appendRoot, nextValue);
        }
        _results.push(setModelValue(modelAdapter, nextRoot, remainder, nextValue, ver));
      }
      return _results;
    }
  };
  populateModel = function(model, paths, callback) {
    var clientId, getting, key, maxVer, modelAdapter, path, remainder, root, value, _i, _j, _len, _len2, _paths, _ref2, _results;
    _paths = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      if (typeof path === 'object') {
        for (key in path) {
          value = path[key];
          root = pathParser.splitPattern(value)[0];
          model.set(key, model.ref(root));
          _paths.push(value);
        }
        continue;
      }
      _paths.push(path);
    }
    model._storeSubs = model._storeSubs.concat(_paths);
    clientId = model._clientId;
    pubSub.subscribe(clientId, _paths);
    maxVer = 0;
    getting = _paths.length;
    modelAdapter = model._adapter;
    _results = [];
    for (_j = 0, _len2 = _paths.length; _j < _len2; _j++) {
      path = _paths[_j];
      _ref2 = pathParser.splitPattern(path), root = _ref2[0], remainder = _ref2[1];
      _results.push(adapter.get(root, function(err, value, ver) {
        var done, onTxn;
        if (err) {
          if (callback) {
            callback(err);
          }
          return callback = null;
        }
        maxVer = Math.max(maxVer, ver);
        setModelValue(modelAdapter, root, remainder, value, ver);
        if (--getting) {
          return;
        }
        modelAdapter.ver = maxVer;
        return forTxnSince(maxVer + 1, clientId, onTxn = function(txn) {
          var args, method;
          method = transaction.method(txn);
          args = transaction.args(txn).slice(0);
          args.push(transaction.base(txn));
          return modelAdapter[method].apply(modelAdapter, args);
        }, done = function() {
          localModels[clientId] = model;
          if (callback) {
            return callback(null, model);
          }
        });
      }));
    }
    return _results;
  };
  this.subscribe = function() {
    var callback, model, paths, waitForStartId, _i;
    model = arguments[0], paths = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    if (arguments.length === 1) {
      callback = model;
    } else {
      if (model instanceof Model) {
        return populateModel(model, paths, callback);
      }
      paths.unshift(model);
    }
    return nextClientId(waitForStartId = function(clientId) {
      if (!startId) {
        return setTimeout(waitForStartId, 10, clientId);
      }
      model = new Model(clientId);
      model.store = self;
      model._ioUri = self._ioUri;
      model._startId = startId;
      return populateModel(model, paths, callback);
    });
  };
  this.unsubscribe = function() {
    var callback, model, modelAdapter, paths, _i;
    model = arguments[0], paths = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    throw new Error('Unimplemented');
    modelAdapter = model._adapter;
    if (!paths.length) {
      return modelAdapter.flush();
    } else {
      ;
    }
  };
  this.flush = function(callback) {
    var cb, done;
    done = false;
    cb = function(err) {
      if (callback && (done || err)) {
        callback(err);
        callback = null;
      }
      return done = true;
    };
    adapter.flush(cb);
    return redisClient.flushdb(function(err) {
      if (err && callback) {
        callback(err);
        return callback = null;
      }
      return redisInfo.onStart(redisClient, cb);
    });
  };
  this.get = function(path, callback) {
    return this._adapter.get(path, callback);
  };
  this.set = function(path, value, ver, callback) {
    return commit(transaction.create({
      base: ver,
      id: nextTxnId(),
      method: 'set',
      args: [path, value]
    }), callback);
  };
  this.del = function(path, ver, callback) {
    return commit(transaction.create({
      base: ver,
      id: nextTxnId(),
      method: 'del',
      args: [path]
    }), callback);
  };
  this.incr = function(path, byNum) {
    if (byNum == null) {
      byNum = 1;
    }
    return this.retry(function(atomic) {
      return atomic.get(path, function(val) {
        return atomic.set(path, (val || 0) + byNum);
      });
    });
  };
  StoreAtomic = function(store, cb) {
    var count, minVer;
    minVer = 0;
    count = 0;
    this._reset = function() {
      minVer = 0;
      return count = 0;
    };
    this.get = function(path, callback) {
      return store.get(path, function(err, value, ver) {
        if (err) {
          return cb(err);
        }
        minVer = minVer ? Math.min(minVer, ver) : ver;
        if (callback) {
          return callback(value);
        }
      });
    };
    this.set = function(path, value, callback) {
      count++;
      return store.set(path, value, minVer, function(err) {
        if (err) {
          return cb(err);
        }
        if (callback) {
          callback(value);
        }
        if (!--count) {
          return cb();
        }
      });
    };
    this.del = function(path, callback) {
      count++;
      return store.del(path, minVer, function(err) {
        if (err) {
          return cb(err);
        }
        if (callback) {
          callback;
        }
        if (!--count) {
          return cb();
        }
      });
    };
  };
  this.retry = function(fn, callback) {
    var atomic, retries;
    retries = MAX_RETRIES;
    atomic = new StoreAtomic(this, function(err) {
      var delay;
      if (!err) {
        return callback && callback();
      }
      if (!retries--) {
        return callback && callback('maxRetries');
      }
      atomic._reset();
      delay = (MAX_RETRIES - retries) * RETRY_DELAY;
      return setTimeout(fn, delay, atomic);
    });
    return fn(atomic);
  };
  nextClientId = function(callback) {
    return redisClient.incr('clientClock', function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value.toString(36));
    });
  };
  clientId = '';
  nextClientId(function(value) {
    return clientId = '#' + value;
  });
  txnCount = 0;
  nextTxnId = function() {
    return clientId + '.' + txnCount++;
  };
  stm = new Stm(redisClient);
  this._commit = commit = function(txn, callback) {
    var ver;
    ver = transaction.base(txn);
    if (ver && typeof ver !== 'number') {
      throw new Error('Version must be null or a number');
    }
    return stm.commit(txn, function(err, ver) {
      transaction.base(txn, ver);
      if (callback) {
        callback(err, txn);
      }
      if (err) {
        return;
      }
      pubSub.publish(transaction.path(txn), txn);
      return txnApplier.add(txn, ver);
    });
  };
  txnApplier = new TxnApplier({
    applyTxn: function(txn, ver) {
      var args, method;
      args = transaction.args(txn).slice(0);
      method = transaction.method(txn);
      args.push(ver, function(err) {
        if (err) {
          throw err;
        }
      });
      return adapter[method].apply(adapter, args);
    }
  });
};

==> racer-0.0.8.tgz: package/lib/Store.js <==
var MAX_RETRIES, MemoryAdapter, Model, PubSub, RETRY_DELAY, Stm, Store, TxnApplier, pathParser, redis, redisInfo, transaction;
var __slice = Array.prototype.slice;
redis = require('redis');
MemoryAdapter = require('./adapters/Memory');
Model = require('./Model');
Stm = require('./Stm');
PubSub = require('./PubSub');
transaction = require('./transaction');
TxnApplier = require('./TxnApplier');
pathParser = require('./pathParser.server');
redisInfo = require('./redisInfo');
MAX_RETRIES = 20;
RETRY_DELAY = 5;
Store = module.exports = function(options) {
  var Adapter, StoreAtomic, adapter, clientId, clientSockets, commit, db, forTxnSince, hasInvalidVer, host, ignoreSubscribe, localModels, nextClientId, nextTxnId, nextTxnNum, populateModel, port, pubSub, redisClient, redisOptions, redisStarts, self, setModelValue, startId, stm, subClient, subscribeToStarts, txnApplier, txnCount, txnSubClient, _ref;
  if (options == null) {
    options = {};
  }
  self = this;
  Adapter = options.Adapter || MemoryAdapter;
  this._adapter = adapter = new Adapter;
  _ref = redisOptions = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db;
  this._redisClient = redisClient = redis.createClient(port, host, redisOptions);
  this._subClient = subClient = redis.createClient(port, host, redisOptions);
  this._txnSubClient = txnSubClient = redis.createClient(port, host, redisOptions);
  redisStarts = null;
  startId = null;
  ignoreSubscribe = false;
  (subscribeToStarts = function(selected) {
    if (ignoreSubscribe) {
      return ignoreSubscribe = false;
    }
    if (db !== void 0 && !selected) {
      return redisClient.select(db, function(err) {
        if (err) {
          throw err;
        }
        return subscribeToStarts(true);
      });
    }
    return redisInfo.subscribeToStarts(subClient, redisClient, function(starts) {
      redisStarts = starts;
      return startId = starts[0][0];
    });
  })();
  ignoreSubscribe = true;
  redisClient.on('connect', subscribeToStarts);
  redisClient.on('end', function() {
    redisStarts = null;
    return startId = null;
  });
  this._localModels = localModels = {};
  this._pubSub = pubSub = new PubSub({
    redis: redisOptions,
    pubClient: redisClient,
    subClient: txnSubClient,
    onMessage: function(clientId, txn) {
      var base, model, socket;
      if (clientId === transaction.clientId(txn)) {
        return;
      }
      if (model = localModels[clientId]) {
        return model._onTxn(txn);
      }
      if (socket = clientSockets[clientId]) {
        base = transaction.base(txn);
        if (base > socket.__base) {
          socket.__base = base;
          return nextTxnNum(clientId, function(num) {
            return socket.emit('txn', txn, num);
          });
        }
      }
    }
  });
  this._nextTxnNum = nextTxnNum = function(clientId, callback) {
    return redisClient.incr('txnClock.' + clientId, function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value);
    });
  };
  hasInvalidVer = function(socket, ver, clientStartId) {
    if (!startId) {
      socket.disconnect();
      return true;
    }
    if (!(clientStartId && clientStartId === startId)) {
      socket.emit('fatalErr');
      return true;
    }
    return false;
  };
  clientSockets = {};
  this._setSockets = function(sockets) {
    return sockets.on('connection', function(socket) {
      return socket.on('sub', function(clientId, paths, ver, clientStartId) {
        var txnsSince;
        if (hasInvalidVer(socket, ver, clientStartId)) {
          return;
        }
        socket.on('disconnect', function() {
          pubSub.unsubscribe(clientId);
          delete clientSockets[clientId];
          return redisClient.del('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
          });
        });
        socket.on('txn', function(txn, clientStartId) {
          var base;
          base = transaction.base(txn);
          if (hasInvalidVer(socket, base, clientStartId)) {
            return;
          }
          return commit(txn, function(err, txn) {
            var txnId;
            txnId = transaction.id(txn);
            base = transaction.base(txn);
            if (err && err !== 'duplicate') {
              return socket.emit('txnErr', err, txnId);
            }
            return nextTxnNum(clientId, function(num) {
              return socket.emit('txnOk', txnId, base, num);
            });
          });
        });
        socket.on('txnsSince', txnsSince = function(ver, clientStartId) {
          if (hasInvalidVer(socket, ver, clientStartId)) {
            return;
          }
          return redisClient.get('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
            socket.emit('txnNum', value || 0);
            return forTxnSince(ver, clientId, function(txn) {
              return nextTxnNum(clientId, function(num) {
                socket.__base = transaction.base(txn);
                return socket.emit('txn', txn, num);
              });
            });
          });
        });
        socket.__base = 0;
        clientSockets[clientId] = socket;
        pubSub.subscribe(clientId, paths);
        return txnsSince(ver + 1, clientStartId);
      });
    });
  };
  this._forTxnSince = forTxnSince = function(ver, clientId, onTxn, done) {
    if (!pubSub.hasSubscriptions(clientId)) {
      return;
    }
    return redisClient.zrangebyscore('txns', ver, '+inf', 'withscores', function(err, vals) {
      var i, txn, val, _len;
      if (err) {
        throw err;
      }
      txn = null;
      for (i = 0, _len = vals.length; i < _len; i++) {
        val = vals[i];
        if (i % 2) {
          if (!pubSub.subscribedToTxn(clientId, txn)) {
            continue;
          }
          transaction.base(txn, +val);
          onTxn(txn);
        } else {
          txn = JSON.parse(val);
        }
      }
      if (done) {
        return done();
      }
    });
  };
  setModelValue = function(modelAdapter, root, remainder, value, ver) {
    var appendRoot, c, nextRoot, nextValue, prop, _ref2, _results;
    if (!remainder) {
      value = typeof value === 'object' ? Array.isArray(value) ? [] : {} : value;
      return modelAdapter.set(root, value, ver);
    }
    if (remainder === '**') {
      return modelAdapter.set(root, value, ver);
    }
    if (remainder.charAt(0) === '*' && (c = remainder.charAt(1)) === '.' || c === '') {
      _ref2 = pathParser.splitPattern(remainder.substr(2)), appendRoot = _ref2[0], remainder = _ref2[1];
      _results = [];
      for (prop in value) {
        nextRoot = root ? root + '.' + prop : prop;
        nextValue = value[prop];
        if (appendRoot) {
          nextRoot += '.' + appendRoot;
          nextValue = pathParser.fastLookup(appendRoot, nextValue);
        }
        _results.push(setModelValue(modelAdapter, nextRoot, remainder, nextValue, ver));
      }
      return _results;
    }
  };
  populateModel = function(model, paths, callback) {
    var clientId, getting, key, maxVer, modelAdapter, path, remainder, root, value, _i, _j, _len, _len2, _paths, _ref2, _results;
    _paths = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      if (typeof path === 'object') {
        for (key in path) {
          value = path[key];
          root = pathParser.splitPattern(value)[0];
          model.set(key, model.ref(root));
          _paths.push(value);
        }
        continue;
      }
      _paths.push(path);
    }
    model._storeSubs = model._storeSubs.concat(_paths);
    clientId = model._clientId;
    pubSub.subscribe(clientId, _paths);
    maxVer = 0;
    getting = _paths.length;
    modelAdapter = model._adapter;
    _results = [];
    for (_j = 0, _len2 = _paths.length; _j < _len2; _j++) {
      path = _paths[_j];
      _ref2 = pathParser.splitPattern(path), root = _ref2[0], remainder = _ref2[1];
      _results.push(adapter.get(root, function(err, value, ver) {
        var done, onTxn;
        if (err) {
          if (callback) {
            callback(err);
          }
          return callback = null;
        }
        maxVer = Math.max(maxVer, ver);
        setModelValue(modelAdapter, root, remainder, value, ver);
        if (--getting) {
          return;
        }
        modelAdapter.ver = maxVer;
        return forTxnSince(maxVer + 1, clientId, onTxn = function(txn) {
          var args, method;
          method = transaction.method(txn);
          args = transaction.args(txn).slice(0);
          args.push(transaction.base(txn));
          return modelAdapter[method].apply(modelAdapter, args);
        }, done = function() {
          localModels[clientId] = model;
          if (callback) {
            return callback(null, model);
          }
        });
      }));
    }
    return _results;
  };
  this.subscribe = function() {
    var callback, model, paths, waitForStartId, _i;
    model = arguments[0], paths = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    if (arguments.length === 1) {
      callback = model;
    } else {
      if (model instanceof Model) {
        return populateModel(model, paths, callback);
      }
      paths.unshift(model);
    }
    return nextClientId(waitForStartId = function(clientId) {
      if (!startId) {
        return setTimeout(waitForStartId, 10, clientId);
      }
      model = new Model(clientId);
      model.store = self;
      model._ioUri = self._ioUri;
      model._startId = startId;
      return populateModel(model, paths, callback);
    });
  };
  this.unsubscribe = function() {
    var callback, model, modelAdapter, paths, _i;
    model = arguments[0], paths = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    throw new Error('Unimplemented');
    modelAdapter = model._adapter;
    if (!paths.length) {
      return modelAdapter.flush();
    } else {
      ;
    }
  };
  this.flush = function(callback) {
    var cb, done;
    done = false;
    cb = function(err) {
      if (callback && (done || err)) {
        callback(err);
        callback = null;
      }
      return done = true;
    };
    adapter.flush(cb);
    return redisClient.flushdb(function(err) {
      if (err && callback) {
        callback(err);
        return callback = null;
      }
      return redisInfo.onStart(redisClient, cb);
    });
  };
  this.get = function(path, callback) {
    return this._adapter.get(path, callback);
  };
  this.set = function(path, value, ver, callback) {
    return commit(transaction.create({
      base: ver,
      id: nextTxnId(),
      method: 'set',
      args: [path, value]
    }), callback);
  };
  this.del = function(path, ver, callback) {
    return commit(transaction.create({
      base: ver,
      id: nextTxnId(),
      method: 'del',
      args: [path]
    }), callback);
  };
  this.incr = function(path, byNum) {
    if (byNum == null) {
      byNum = 1;
    }
    return this.retry(function(atomic) {
      return atomic.get(path, function(val) {
        return atomic.set(path, (val || 0) + byNum);
      });
    });
  };
  StoreAtomic = function(store, cb) {
    var count, minVer;
    minVer = 0;
    count = 0;
    this._reset = function() {
      minVer = 0;
      return count = 0;
    };
    this.get = function(path, callback) {
      return store.get(path, function(err, value, ver) {
        if (err) {
          return cb(err);
        }
        minVer = minVer ? Math.min(minVer, ver) : ver;
        if (callback) {
          return callback(value);
        }
      });
    };
    this.set = function(path, value, callback) {
      count++;
      return store.set(path, value, minVer, function(err) {
        if (err) {
          return cb(err);
        }
        if (callback) {
          callback(value);
        }
        if (!--count) {
          return cb();
        }
      });
    };
    this.del = function(path, callback) {
      count++;
      return store.del(path, minVer, function(err) {
        if (err) {
          return cb(err);
        }
        if (callback) {
          callback;
        }
        if (!--count) {
          return cb();
        }
      });
    };
  };
  this.retry = function(fn, callback) {
    var atomic, retries;
    retries = MAX_RETRIES;
    atomic = new StoreAtomic(this, function(err) {
      var delay;
      if (!err) {
        return callback && callback();
      }
      if (!retries--) {
        return callback && callback('maxRetries');
      }
      atomic._reset();
      delay = (MAX_RETRIES - retries) * RETRY_DELAY;
      return setTimeout(fn, delay, atomic);
    });
    return fn(atomic);
  };
  nextClientId = function(callback) {
    return redisClient.incr('clientClock', function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value.toString(36));
    });
  };
  clientId = '';
  nextClientId(function(value) {
    return clientId = '#' + value;
  });
  txnCount = 0;
  nextTxnId = function() {
    return clientId + '.' + txnCount++;
  };
  stm = new Stm(redisClient);
  this._commit = commit = function(txn, callback) {
    var ver;
    ver = transaction.base(txn);
    if (ver && typeof ver !== 'number') {
      throw new Error('Version must be null or a number');
    }
    return stm.commit(txn, function(err, ver) {
      transaction.base(txn, ver);
      if (callback) {
        callback(err, txn);
      }
      if (err) {
        return;
      }
      pubSub.publish(transaction.path(txn), txn);
      return txnApplier.add(txn, ver);
    });
  };
  txnApplier = new TxnApplier({
    applyTxn: function(txn, ver) {
      var args, method;
      args = transaction.args(txn).slice(0);
      method = transaction.method(txn);
      args.push(ver, function(err) {
        if (err) {
          throw err;
        }
      });
      return adapter[method].apply(adapter, args);
    }
  });
};
